// dblltest.c
//
// when delete-ing very large lists the de-allocation routine free() in
// crtdll.dll is very slow. To over-come this problem use the msvcrt.dll
//
// this is achieved by linking with msvcrt.lib
//
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dbll.h"

int compare( const void * a, const void * b)
{
	// to sort in reverse order use -strcmp( a, b )
	return strcmp( a, b );
}

int main( void )
{
	DBLL root, newnode, node;
	char s[20];
	int t, ii, i;

	// the pointers to these strings will be stored in the the linked list.
	char mmm[] = "C Jumps over";
	char mm[2][12] = {"B A quick", "A brown fox"};
	char m[5][10] = {"D Then", "F Goodbye", "E Today", "G Hello", "C Now"};

	// create the linked list's root node
	if (NULL == (root = dbll_new())){
		printf("No Memory today!\n");
		return 0;
	}

	node = root;

	// append the 5 string from char array m[]
	for (i = 0; i<5; i++){
		if (NULL == (node = dbll_append(node, m[i]))){
			printf("Opps! No more memory!\n");
			dbll_delete_all( node );
			return 0;
		}
	}

	// print all, start with the first data node. dbll_getnext( root )
	// will return the first data node
	node = root;
	while( (node = dbll_getnext( node )) ){
		printf("%s %d\n", dbll_getdata( node ), dbll_getindex( node ));
	};

	printf("COUNT %d\n", dbll_count( root ));
	printf("%\n");

	node = dbll_getnext( root ); 					// get a valid node
	newnode = dbll_insert( node, mm[0], 2);			// insert mm[0] at index 2
	printf("COUNT %d\n", dbll_count( root ));

	// get the first data node into newnode
	newnode = dbll_getfirst( root );
	printf("Get first node -> %s\n", dbll_getdata( newnode ));
	newnode = dbll_getlast( root );
	printf("Get last node -> %s\n", dbll_getdata( newnode ));

	// insert a new data node before 'node'
	newnode = dbll_prepend( node, mm[1] );
	printf("COUNT %d\n", dbll_count( root ));
	printf("%\n");

	// print strings in reverse order
	node = dbll_getlast( node );
	// 'node' now points to the last node
	while( dbll_getprev( node )){
		printf("%s %d\n", dbll_getdata( node ), dbll_getindex( node ));
		node = dbll_getprev( node );
	};

	// node now is at the root node, shuffle node on
	node = dbll_getnext( node );
	node = dbll_getnext( node );

	printf("%\n");
	node    = dbll_getnode(node, 2);  	// get the node with an index of 2
	printf("Second node value -> %s %d\n", dbll_getdata( node ), dbll_getindex( node ));
	printf("%\n");

	// sort the list
	node = dbll_msort( node, compare );

	// print all sorted strings
	node = dbll_getfirst( root );		// start node at the beginning again
	while( dbll_getnext( node ) ){
		printf("Sorted %d %s\n", dbll_getindex( node ), dbll_getdata( node ));
		node = dbll_getnext( node );
	}
	// print last one
	printf("Sorted %d %s\n", dbll_getindex( node ), dbll_getdata( node ));
	printf("COUNT %d\n", dbll_count( root ));

	// now insert a data node using dbll_insert_sorted()
	node = dbll_getprev(node);
	node = dbll_insert_sorted( node, mmm, compare );

	// print them
	node = dbll_getfirst( root );			// start node at the beginning again
	while( dbll_getnext( node ) ){
		printf("Sorted %d %s\n", dbll_getindex( node ), dbll_getdata( node ));
		node = dbll_getnext( node );
	}
	// print last one
	printf("Sorted %d %s\n", dbll_getindex( node ), dbll_getdata( node ));
	printf("COUNT %d\n", dbll_count( root ));

	// cleanup
	dbll_delete_all( root );

	char st[20];
	printf("Press Return\n");
	fgets(st, 10, stdin);
	return 0;
}
