#include <windows.h>
#include <stdio.h>
#include <gc.h>

// Link with jlib.lib and gc.lib

#include "jmem.h"

#define NUM 250000
char * s[NUM];


int main(void)
{
	DWORD t = GetTickCount();

	for (int i = 0; i<NUM; i++){
		s[i] = malloc(20);
	}

	DWORD tt = GetTickCount();
	printf( "Time to malloc 250000      %d ms\n", tt - t );

	t = GetTickCount();
	for (int i = 0; i<NUM; i++){
		free(s[i]);
	}

	tt = GetTickCount();
	printf( "Time to free 250000        %d ms\n", tt - t );

	HMEM hmem = InitJMem( 200000, 140 );
	t = GetTickCount();

	for (int i = 0; i<NUM; i++){
		s[i] = jmalloc( hmem, 80 );
	}

	tt = GetTickCount();
	printf( "Time to jmalloc 250000     %d ms\n", tt - t );

	printf( "Current block in use       %d\n", get_block( hmem ));
	printf( "Number of blocks           %d\n", get_numblocks( hmem ));

	t = GetTickCount();

	jfree( hmem );

	tt = GetTickCount();
	printf( "Time to jfree 250000       %d ms\n", tt - t );

	t = GetTickCount();
	for (int i = 0; i<NUM; i++){
		s[i] = GC_malloc( 80 );
	}

	tt = GetTickCount();
	printf( "Time to GC_malloc 250000   %d ms\n", tt - t );

	printf( "\n" );

	// demo of increasing the number of blocks
	// make a new mem object
	HMEM hmem1 = InitJMem( 20000, 14 );
	printf( "Number of blocks -> %d\n", get_numblocks( hmem1 ));

	// add blocks
	HMEM tmp = add_blocks( hmem1, 25 );
	// if successful assign tmp to hmem1
	if (tmp)
		hmem1 = tmp;

	printf( "Number of blocks -> %d\n", get_numblocks( hmem1 ));

	jfree( hmem1 );


//	jmem_libvers();
	return 0;
}
