// queuetest.c demonstrating various stack funcs
// Link with Complex.lib & jlib.lib

#include <stdlib.h>  // for sleep()
#include <stdio.h>   // for printf
#include <limits.h>  // for ULONG_MAX
#include <complex.h> // complex stuff
#include <float.h>   // for DBL_MAX etc.
#include "queue.h"


int main(void)
{
	LQUEUE l;						// create a stack for long's
	int cnt = 0;
	if (LQueueInit( l, 10 )){

		while ( !l.lisqfull( l ) ){
			l.lput( l, cnt++ );
		}

		xprintf("NumItems for long queue %d\n", l.litems( l ) );

		long lvar;
		lvar = l.lget( l );
		lvar = l.lget( l );
		l.lput( l, 333 );

		while ( !l.lisqempty( l ) ){
			lvar = l.lget( l );		// get items until empty
			printf( "%u\n", lvar );
		}

		l.lclear( l );				// example of cleaning stack
		l.lerase( l );				// cleanup the stack
	}

	sleep(2000);

	LLQUEUE ll;						// create a stack for long long's
	cnt = 0;
	if (LLQueueInit( ll, 10 )){

		while ( !ll.llisqfull( ll ) ){
			ll.llput( ll, cnt++ );
		}

		xprintf("NumItems for long long queue %d\n", ll.llitems( ll ) );

		long long llvar;
		llvar = ll.llget( ll );
		llvar = ll.llget( ll );
		llvar = ll.llget( ll );
		ll.llput( ll, 333 );

		while ( !ll.llisqempty( ll ) ){
			llvar = ll.llget( ll );		// get items until empty
			printf( "%u\n", llvar );
		}

		ll.llclear( ll );				// example of cleaning stack
		ll.llerase( ll );				// cleanup the stack

	}

	sleep(2000);

	DQUEUE d;						// create a stack for double's

	double dcnt = 1.1f;

	if (DQueueInit( d, 20 )){

		while ( !d.disqfull( d ) ){
			d.dput( d, dcnt );
			dcnt += 0.1f;
		}

		xprintf("NumItems for double queue %d\n", d.ditems( d ) );

		double dvar;
		dvar = d.dget( d );
		dvar = d.dget( d );
		dvar = d.dget( d );
		d.dput( d, 333.1f );

		while ( !d.disqempty( d ) ){
			dvar = d.dget( d );		// get items until empty
			printf( "%g\n", dvar );
		}

		d.dclear( d );
		d.derase( d );
	}

	sleep(2000);

	FQUEUE f;						// create a stack for float's

	if (FQueueInit( f, 20 )){

		float fcnt = 1.1f;

		while ( !f.fisqfull( f ) ){
			f.fput( f, fcnt );
			fcnt += 0.1f;
		}

		xprintf("NumItems for float queue %d\n", f.fitems( f ) );

		float fvar;
		fvar = f.fget( f );
		fvar = f.fget( f );
		fvar = f.fget( f );
		f.fput( f, 333.1f );

		while ( !f.fisqempty( f ) ){
			fvar = f.fget( f );		// get items until empty
			printf( "%g\n", fvar );
		}

		f.fclear( f );
		f.ferase( f );
	}

	sleep(2000);

	LDQUEUE ld;						// create a stack for long double's

	if (LDQueueInit( ld, 20 )){

		long double ldcnt = 1.1f;

		while ( !ld.ldisqfull( ld ) ){
			ld.ldput( ld, ldcnt );
			ldcnt += 0.1f;
		}

		xprintf("NumItems for long double queue %d\n", ld.lditems( ld ) );

		long double ldvar;
		ldvar = ld.ldget( ld );
		ldvar = ld.ldget( ld );
		ldvar = ld.ldget( ld );
		ld.ldput( ld, 333.1f );

		while ( !ld.ldisqempty( ld ) ){
			ldvar = ld.ldget( ld );		// get items until empty
			xprintf( "%Lg\n", ldvar );
		}

		ld.ldclear( ld );
		ld.lderase( ld );

	}

	sleep(2000);

	PQUEUE p;						// create a stack for pointers

	if (PQueueInit( p, 20 )){

		int * pcnt = (int*)0x45d80;

		while ( !p.pisqfull( p ) ){
			p.pput( p, pcnt );
			pcnt += 1;
		}

		xprintf("NumItems for generic pointer queue %d\n", p.pitems( p ) );

		int *  pvar;
		pvar = p.pget( p );
		pvar = p.pget( p );
		pvar = p.pget( p );
		p.pput( p, (int*)0xfab60 );

		while ( !p.pisqempty( p ) ){
			pvar = p.pget( p );		// get items until empty
			printf( "%p\n", pvar );
		}

		p.pclear( p );
		p.perase( p );
	}

	sleep(2000);

	DCQUEUE dc;						// create a stack for double _Complex

	double _Complex dcvar, dcvar1;

	dcvar1.re = 1.2345; 	dcvar1.im = 1.2345;

	dcvar = ccos(dcvar1);

	if (DCQueueInit( dc, 20 )){

		int * pcnt = (int*)0x45d80;

		while ( !dc.dcisqfull( dc ) ){
			dc.dcput( dc, dcvar );
			dcvar += dcvar1;
		}

		xprintf("NumItems for double _Complex queue %d\n", dc.dcitems( dc ) );

		double _Complex dcvar3;
		dcvar3 = dc.dcget( dc );
		dcvar3 = dc.dcget( dc );
		dcvar3 = dc.dcget( dc );
		dc.dcput( dc, dcvar1 );

		while ( !dc.dcisqempty( dc ) ){
			dcvar = dc.dcget( dc );		// get items until empty
			printf( "%Lg  %Lg\n", dcvar.re, dcvar.im );
		}

		dc.dcclear( dc );
		dc.dcerase( dc );
	}

	printf("Press enter\n");

//	queue_libvers();
	char m[20];
	fgets(m, 10, stdin);
	return 0;

}

