// queue.h

// interface for 'long queue'
#include <math.h>
typedef struct lqu LQUEUE;

// prototypes for long queue functions
void lput(LQUEUE&, long);
long lget(LQUEUE&);
int  lclear(LQUEUE&);
int  lisqempty(LQUEUE&);
int  lisqfull(LQUEUE&);
void lerase(LQUEUE&);
int  litems(LQUEUE&);
int  LQueueInit(LQUEUE &d, int n);
void queue_libvers();

typedef struct lqu
{	int    size;							// allocated size. i.e. 20 longs
	int    items;							// how many items in use
	long  * head;							// top of queue
	long  * tail;							// bottom of queue
	long  * slide;							//
	void   (*lput)(LQUEUE&, long);
	long   (*lget)(LQUEUE&);				// will point to the various
	int    (*lclear)(LQUEUE&);
	int    (*lisqempty)(LQUEUE&);
	int    (*lisqfull)(LQUEUE&);
	void   (*lerase)(LQUEUE&);				// functions after initialising
	int    (*litems)(LQUEUE&);			// the type
}LQUEUE;

// interface for 'double queue'
typedef struct dqu DQUEUE;

void    dput(DQUEUE&, double);
double  dget(DQUEUE&);
int     dclear(DQUEUE&);
int     disqempty(DQUEUE&);
int     disqfull(DQUEUE&);
void    derase(DQUEUE&);
int     ditems(DQUEUE&);
int     DQueueInit(DQUEUE &d, int n);

typedef struct dqu
{	int    size;
	int    items;
	double * head;
	double * tail;
	double * slide;
	void   (*dput)(DQUEUE&, double);
	double (*dget)(DQUEUE&);
	int    (*dclear)(DQUEUE&);
	int    (*disqempty)(DQUEUE&);
	int    (*disqfull)(DQUEUE&);
	void   (*derase)(DQUEUE&);
	int    (*ditems)(DQUEUE&);
}DQUEUE;


// interface for 'float queue'
typedef struct fqu FQUEUE;

void    fput(FQUEUE&, double);
float   fget(FQUEUE&);
int     fclear(FQUEUE&);
int     fisqempty(FQUEUE&);
int     fisqfull(FQUEUE&);
void    ferase(FQUEUE&);
int     fitems(FQUEUE&);
int     FQueueInit(FQUEUE &d, int n);

typedef struct fqu
{	int    size;
	int    items;
	float  * head;
	float  * tail;
	float  * slide;
	void   (*fput)(FQUEUE&, double);
	float  (*fget)(FQUEUE&);
	int    (*fclear)(FQUEUE&);
	int    (*fisqempty)(FQUEUE&);
	int    (*fisqfull)(FQUEUE&);
	void   (*ferase)(FQUEUE&);
	int    (*fitems)(FQUEUE&);
}FQUEUE;

// interface for 'long long queue'
typedef struct llqu LLQUEUE;

void      llput(LLQUEUE&, long long);
long long llget(LLQUEUE&);
int       llclear(LLQUEUE&);
int       llisqempty(LLQUEUE&);
int       llisqfull(LLQUEUE&);
void      llerase(LLQUEUE&);
int       llitems(LLQUEUE&);
int       LLQueueInit(LLQUEUE &d, int n);

typedef struct llqu
{	int       size;
	int       items;
	long long * head;
	long long * tail;
	long long * slide;
	void      (*llput)(LLQUEUE&, long long);
	long long (*llget)(LLQUEUE&);
	int       (*llclear)(LLQUEUE&);
	int       (*llisqempty)(LLQUEUE&);
	int       (*llisqfull)(LLQUEUE&);
	void      (*llerase)(LLQUEUE&);
	int       (*llitems)(LLQUEUE&);
}LLQUEUE;

// interface for 'long double queue'
typedef struct ldqu LDQUEUE;

void        ldput(LDQUEUE&, long double);
long double ldget(LDQUEUE&);
int         ldclear(LDQUEUE&);
int         ldisqempty(LDQUEUE&);
int         ldisqfull(LDQUEUE&);
void        lderase(LDQUEUE&);
int         lditems(LDQUEUE&);
int         LDQueueInit(LDQUEUE &d, int n);

typedef struct ldqu
{	int          size;
	int          items;
	long double * head;
	long double * tail;
	long double * slide;
	void        (*ldput)(LDQUEUE&, long double);
	long double (*ldget)(LDQUEUE&);
	int         (*ldclear)(LDQUEUE&);
	int         (*ldisqempty)(LDQUEUE&);
	int         (*ldisqfull)(LDQUEUE&);
	void        (*lderase)(LDQUEUE&);
	int         (*lditems)(LDQUEUE&);
}LDQUEUE;

// interface for 'pointers queue'
typedef struct pqu PQUEUE;

void      pput(PQUEUE&, void *);
void *    pget(PQUEUE&);
int       pclear(PQUEUE&);
int       pisqempty(PQUEUE&);
int       pisqfull(PQUEUE&);
void      perase(PQUEUE&);
int       pitems(PQUEUE&);
int       PQueueInit(PQUEUE &d, int n);

typedef struct pqu
{	int     size;
	int     items;
	void ** head;
	void ** tail;
	void ** slide;
	void    (*pput)(PQUEUE&, void *);
	void *  (*pget)(PQUEUE&);
	int     (*pclear)(PQUEUE&);
	int     (*pisqempty)(PQUEUE&);
	int     (*pisqfull)(PQUEUE&);
	void    (*perase)(PQUEUE&);
	int     (*pitems)(PQUEUE&);
}PQUEUE;

// interface for 'double _Complex queue'
typedef struct dcqu DCQUEUE;

void      dcput(DCQUEUE&, double _Complex);
double    _Complex dcget(DCQUEUE&);
int       dcclear(DCQUEUE&);
int       dcisqempty(DCQUEUE&);
int       dcisqfull(DCQUEUE&);
void      dcerase(DCQUEUE&);
int       dcitems(DCQUEUE&);
int       DCQueueInit(DCQUEUE &d, int n);

typedef struct dcqu
{	int    size;
	int    items;
	double _Complex * head;
	double _Complex * tail;
	double _Complex * slide;
	void   (*dcput)(DCQUEUE&, double _Complex);
	double _Complex (*dcget)(DCQUEUE&);
	int    (*dcclear)(DCQUEUE&);
	int    (*dcisqempty)(DCQUEUE&);
	int    (*dcisqfull)(DCQUEUE&);
	void   (*dcerase)(DCQUEUE&);
	int    (*dcitems)(DCQUEUE&);
}DCQUEUE;

