// stacktest.c demonstrating various stack funcs
// Link with Complex.lib & jlib.lib

#include <stdio.h>   // for printf
#include <limits.h>  // for ULONG_MAX
#include <complex.h> // complex stuff
#include <float.h>   // for DBL_MAX etc.
#include "stack.h"

int main(void)
{
	LSTACK l;						// create a stack for long's

	unsigned long lcnt = 0L;

	if (LStackInit( l, 10 )){

		while ( !l.lisfull( l ) ){
			l.lpush( l, lcnt++ );
		}

		xprintf("NumItems for long stack %d\n", l.lnumitem( l ) );

		unsigned long lvar;

		while ( !l.lisempty( l ) ){
			lvar = l.lpop( l );		// pop items until empty
			printf( "%u\n", lvar );
		}

		l.lpurge( l );				// example of cleaning stack
		l.ldelete( l );				// cleanup the stack
	}

	DSTACK d;						// create a stack for double's

	if (DStackInit( d, 20 )){

		d.dpush( d, 1.23456789 );

		xprintf("NumItems for double stack %d\n", d.dnumitem( d ) );

		double dvar = d.dpop( d );

		xprintf("Value %.20g\n", dvar );

		d.dpurge( d );
		d.ddelete( d );
	}

	FSTACK f;						// create a stack for float's

	if (FStackInit( f, 20 )){

		f.fpush( f, 1.23456789 );

		float fvar = f.fpop( f );

		xprintf("Value %.20g\n", fvar );

		f.fpurge( f );
		f.fdelete( f );
	}

	LLSTACK ll;						// create a stack for long long's

	if (LLStackInit( ll, 20 )){

		ll.llpush( ll, ULLONGMAX );

		unsigned long long llvar = ll.llpop( ll );

		xprintf("Value %Lu\n", llvar );

		ll.llpurge( ll );
		ll.lldelete( ll );
	}

	LDSTACK ld;						// create a stack for long double's

	if (LDStackInit( ld, 20 )){

		ld.ldpush( ld, 1.23456789 );

		long double ldvar = ld.ldpop( ld );

		xprintf("Value %.20Lg\n", ldvar );

		ld.ldpurge( ld );
		ld.lddelete( ld );
	}

	PSTACK p;						// create a stack for pointers

	if (PStackInit( p, 20 )){

		p.ppush( p, (char*)12345678 );

		void * pvar = p.ppop( p );

		xprintf("Value %p\n", pvar );

		p.ppurge( p );
		p.pdelete( p );
	}

	DCSTACK dc;						// create a stack for double _Complex

	double _Complex var, var1;

	var1.re = 1.2345; 	var1.im = 1.2345;

	var = ccos(var1);

	if (DCStackInit( dc, 20 )){

		dc.dcpush( dc, var );

		double _Complex dcvar = dc.dcpop( dc );

		xprintf("Value Real %g Imaginary %g\n", dcvar.re, dcvar.im );

		dc.dcpurge( dc );
		dc.dcdelete( dc );
	}

//	stack_libvers();

	printf("Press enter\n");
	char m[20];
	fgets(m, 10, stdin);
	return 0;
}

