// stack.h

// interface for 'long stack'
#include <math.h>
typedef struct lst LSTACK;

// prototypes for long stack functions
void lpush(LSTACK&, long);
long lpop(LSTACK&);
int  lpurge(LSTACK&);
int  lisempty(LSTACK&);
int  lisfull(LSTACK&);
void ldelete(LSTACK&);
int  lnumitem(LSTACK&);
int  LStackInit(LSTACK &d, int n);
void stack_libvers();

typedef struct lst
{	int    size;							// allocated size. i.e. 20 longs
	int    numitem;							// how many items in use
	long  * head;							// top of stack
	long  * tail;							// bottom of stack
	void   (*lpush)(LSTACK&, long);
	long   (*lpop)(LSTACK&);				// will point to the various
	int    (*lpurge)(LSTACK&);
	int    (*lisempty)(LSTACK&);
	int    (*lisfull)(LSTACK&);
	void   (*ldelete)(LSTACK&);				// functions after initialising
	int    (*lnumitem)(LSTACK&);			// the type
}LSTACK;

// interface for 'double stack'
typedef struct dst DSTACK;

void    dpush(DSTACK&, double);
double  dpop(DSTACK&);
int     dpurge(DSTACK&);
int     disempty(DSTACK&);
int     disfull(DSTACK&);
void    ddelete(DSTACK&);
int     dnumitem(DSTACK&);
int     DStackInit(DSTACK &d, int n);

typedef struct dst
{	int    size;
	int    numitem;
	double * head;
	double * tail;
	void   (*dpush)(DSTACK&, double);
	double (*dpop)(DSTACK&);
	int    (*dpurge)(DSTACK&);
	int    (*disempty)(DSTACK&);
	int    (*disfull)(DSTACK&);
	void   (*ddelete)(DSTACK&);
	int    (*dnumitem)(DSTACK&);
}DSTACK;


// interface for 'float stack'
typedef struct fst FSTACK;

void    fpush(FSTACK&, double);
float   fpop(FSTACK&);
int     fpurge(FSTACK&);
int     fisempty(FSTACK&);
int     fisfull(FSTACK&);
void    fdelete(FSTACK&);
int     fnumitem(FSTACK&);
int     FStackInit(FSTACK &d, int n);

typedef struct fst
{	int    size;
	int    numitem;
	float  * head;
	float  * tail;
	void   (*fpush)(FSTACK&, double);
	float  (*fpop)(FSTACK&);
	int    (*fpurge)(FSTACK&);
	int    (*fisempty)(FSTACK&);
	int    (*fisfull)(FSTACK&);
	void   (*fdelete)(FSTACK&);
	int    (*fnumitem)(FSTACK&);
}FSTACK;

// interface for 'long long stack'
typedef struct llst LLSTACK;

void      llpush(LLSTACK&, long long);
long long llpop(LLSTACK&);
int       llpurge(LLSTACK&);
int       llisempty(LLSTACK&);
int       llisfull(LLSTACK&);
void      lldelete(LLSTACK&);
int       llnumitem(LLSTACK&);
int       LLStackInit(LLSTACK &d, int n);

typedef struct llst
{	int       size;
	int       numitem;
	long long * head;
	long long * tail;
	void      (*llpush)(LLSTACK&, long long);
	long long (*llpop)(LLSTACK&);
	int       (*llpurge)(LLSTACK&);
	int       (*llisempty)(LLSTACK&);
	int       (*llisfull)(LLSTACK&);
	void      (*lldelete)(LLSTACK&);
	int       (*llnumitem)(LLSTACK&);
}LLSTACK;

// interface for 'long double stack'
typedef struct ldst LDSTACK;

void        ldpush(LDSTACK&, long double);
long double ldpop(LDSTACK&);
int         ldpurge(LDSTACK&);
int         ldisempty(LDSTACK&);
int         ldisfull(LDSTACK&);
void        lddelete(LDSTACK&);
int         ldnumitem(LDSTACK&);
int         LDStackInit(LDSTACK &d, int n);

typedef struct ldst
{	int          size;
	int          numitem;
	long double * head;
	long double * tail;
	void        (*ldpush)(LDSTACK&, long double);
	long double (*ldpop)(LDSTACK&);
	int         (*ldpurge)(LDSTACK&);
	int         (*ldisempty)(LDSTACK&);
	int         (*ldisfull)(LDSTACK&);
	void        (*lddelete)(LDSTACK&);
	int         (*ldnumitem)(LDSTACK&);
}LDSTACK;

// interface for 'pointers stack'
typedef struct pst PSTACK;

void      ppush(PSTACK&, void *);
void *    ppop(PSTACK&);
int       ppurge(PSTACK&);
int       pisempty(PSTACK&);
int       pisfull(PSTACK&);
void      pdelete(PSTACK&);
int       pnumitem(PSTACK&);
int       PStackInit(PSTACK &d, int n);

typedef struct pst
{	int     size;
	int     numitem;
	void ** head;
	void ** tail;
	void    (*ppush)(PSTACK&, void *);
	void *  (*ppop)(PSTACK&);
	int     (*ppurge)(PSTACK&);
	int     (*pisempty)(PSTACK&);
	int     (*pisfull)(PSTACK&);
	void    (*pdelete)(PSTACK&);
	int     (*pnumitem)(PSTACK&);
}PSTACK;

// interface for 'double _Complex stack'
typedef struct dcst DCSTACK;

void      dcpush(DCSTACK&, double _Complex);
double    _Complex dcpop(DCSTACK&);
int       dcpurge(DCSTACK&);
int       dcisempty(DCSTACK&);
int       dcisfull(DCSTACK&);
void      dcdelete(DCSTACK&);
int       dcnumitem(DCSTACK&);
int       DCStackInit(DCSTACK &d, int n);

typedef struct dcst
{	int    size;
	int    numitem;
	double _Complex * head;
	double _Complex * tail;
	void   (*dcpush)(DCSTACK&, double _Complex);
	double _Complex (*dcpop)(DCSTACK&);
	int    (*dcpurge)(DCSTACK&);
	int    (*dcisempty)(DCSTACK&);
	int    (*dcisfull)(DCSTACK&);
	void   (*dcdelete)(DCSTACK&);
	int    (*dcnumitem)(DCSTACK&);
}DCSTACK;

