//***********************************************************
//
// str.h
//
// You must have the LCC-Win32 distribution of 27th Nov 2001
// or later for the string library to work.
//
// Johh at <John.Findlay1@btinternet.com>
//
//***********************************************************
#include <windows.h>
#include <stdio.h>

#ifndef _STR_H_
#define _STR_H_

// Abstract data type used for the string object.
// Note: STRING is a pointer to Type.
typedef struct tagString{
	char dummy[1];
}*STRING;

// Possible error codes retrun from various
// library functions
#define STRERR_NONE       0
#define STRERR_MEMORY     -1
#define STRERR_SIZE       -2
#define STRERR_NOTFOUND   -3
#define STRERR_MAXALLOCAT -4
#define STRERR_WRONGARGS  -5
#define STRERR_NOSTRING   -6
#define STRERR_READ       -7
#define STRERR_WRITE      -8

#define NEW str_new( 0 )

// String Functions
// Many functions use the 'overloaded' extension LCC-Win32

BOOL str_isstring( STRING This );

STRING str_dup( STRING restrict s );
STRING str_idreap( int id );
STRING overloaded str_new( int buf_len );
STRING overloaded str_new( const char * c );
STRING str_rev( STRING restrict s );
STRING str_set( STRING restrict s, const char c );

unsigned long overloaded str_hash( const char * restrict src );
unsigned long overloaded str_hash( const STRING restrict src );
unsigned long str_hashget( const STRING restrict src );

int str_fprint( FILE * stream, const STRING s, BOOL nl = TRUE );
int str_format( STRING restrict s, char * format, ... );
int str_fread( FILE *stream, STRING restrict dest, int size );
int str_fwrite( FILE *stream, STRING restrict src, int size );
int str_fgets( FILE *stream, STRING s, int n );
int str_fputs( FILE *stream, const STRING restrict s );

BOOL overloaded str_search( const STRING pattern, int * index, int * pos );
BOOL overloaded str_search( const char * pattern, int * index, int * pos );
BOOL overloaded str_searchi( const STRING pattern, int * index, int * pos );
BOOL overloaded str_searchi( const char * pattern, int * index, int * pos );

int str_print( const STRING restrict s, BOOL nl = TRUE );
int str_printc( const char * s, BOOL nl = TRUE );

int str_printf( const STRING restrict s, char * format, ... );

int  str_qsort( STRING * restrict list, int nelem );
int  str_qsorti( STRING * restrict list, int nelem );

void str_libvers();
void str_delete_all(void);
void str_clean( STRING restrict s );
void str_delete( STRING restrict s );

BOOL overloaded str_isequal( const STRING restrict s1, const STRING restrict s2 );
BOOL overloaded str_isequal( const STRING restrict s1, const char * c );

BOOL overloaded str_isequali( const STRING restrict s1, const STRING restrict s2 );
BOOL overloaded str_isequali( const STRING restrict s1, const char * c );

int str_tab_to_space( STRING s, int numspaces );
int str_getid( const STRING restrict s );

int overloaded str_remspace( STRING src );
int overloaded str_remspace( char * c );

int overloaded str_extractpath( STRING restrict dest, const STRING restrict src );
int overloaded str_extractpath( STRING restrict dest, const char * c );

int overloaded str_extractfile( STRING restrict dest, const STRING restrict src );
int overloaded str_extractfile( STRING restrict dest, const char * c );

int overloaded str_insert( STRING restrict dest, const char * c, int pos, int nlen );
int overloaded str_insert( STRING restrict dest, const STRING restrict src, int pos, int nlen );

int overloaded str_add( STRING restrict dest, const STRING restrict src );
int overloaded str_add( STRING restrict dest, const char * c );

int overloaded str_copy( STRING restrict dest, const char * c );
int overloaded str_copy( STRING restrict dest, const STRING restrict src );

int  str_count();

int str_float( STRING restrict s, const double f );
int str_int( STRING restrict s, int i );

int str_len( const STRING restrict s );

int overloaded str_extract(STRING restrict dest, const STRING restrict src, int pos, int nlen );
int overloaded str_extract(STRING restrict dest, const char * c, int pos, int nlen );

int overloaded str_locate(const STRING restrict s1, const STRING restrict s2 );
int overloaded str_locate(const STRING restrict s1, const char *c );

int overloaded str_locatei(const STRING restrict s1, const STRING restrict s2 );
int overloaded str_locatei(const STRING restrict s1, const char *c );

int str_upper( STRING restrict s );
int str_lower( STRING restrict s );
int str_instr( STRING restrict s1, const char c, int pos );
int str_rinstr( STRING restrict s1, const char c, int pos );
int str_msgbox( const STRING restrict s, const STRING restrict title, UINT uType );

char * str_arrptr( STRING restrict s);
int    str_resize( STRING restrict s, int size );

char * str_to_char( const STRING restrict s );

int overloaded str_token( STRING restrict dest, const STRING restrict src, const STRING restrict control );
int overloaded str_token( STRING restrict dest, const STRING restrict src, const char * control );
int overloaded str_token( STRING restrict dest, const char * src, const char * control );

int str_lasterror();

int overloaded str_cmp( const STRING restrict s1, const STRING restrict s2 );
int overloaded str_cmp( const STRING restrict s1, const char * c );

int str_buflen( const STRING restrict s );

int overloaded str_cmpi( const STRING restrict s1, const char * c );
int overloaded str_cmpi( const STRING restrict s1, const STRING restrict s2 );

int overloaded str_xlate( STRING src, char * table );
int overloaded str_xlate( char * src, char * table );

#endif
