//*************************************************************
//
// teststr.c
//
// Many of the string library functions are used in this example.
//
// Johh at <John.Findlay1@btinternet.com>
//
// You must have the LCC-Win32 distribution of 27th Nov 2001
// or later for using the jlib library.
//
//*************************************************************

#include <windows.h>
#include "str.h"	// for the string library. Link with jlib.lib

int main(void)
{

	STRING a = str_new( 0 );
	STRING b = str_new( 2000 );
	STRING c = str_new( "Qwerty" );
	STRING d = str_new( "" );

	str_printf( NULL, "%d\n", str_isstring( a ) );

	str_copy( a, "A quick brown fox " );
	STRING e = a + "jumps over the lazy dog ";
	str_print( e );
	STRING f = a + e;
	str_print( f );
	STRING k = d + "and creates confusion";
	STRING l = a + e + k + " zimmer";
	str_print( k );
	str_print( l );

	str_rev( a );
	str_upper( a );
	str_print( a, TRUE );

	str_copy( a, "abc,d,efgh,jkl" );
	int n;

	n = str_token( b, a, "," );
	if ( n )
		str_print( b, TRUE );

	while (n = str_token( b, NULL, "," ) ){
		str_print( b, TRUE );
	}

	str_copy( b, "brown" );
	str_copy( c, "The quick brown fox went" );
	str_copy( d, " jumped over the moon" );

	str_printf( NULL, "%d\n", str_locate( c, "fox" ) );
	str_printf( NULL, "%d\n", str_locate( c, b ) );
	str_printf( NULL, "%d\n", str_locatei( c, "FOX" ) );

	str_add( b, " went" );
	str_print( b, TRUE );
	str_add( b, c );
	str_print( b, TRUE );

	str_print( d, TRUE );
	str_copy( d, b);
	str_print( d );

	str_insert( d, "QWERTY", 2, 6 );
	str_print( d, TRUE );
	str_copy( b, "Qwerty" );
	str_insert( d, b, 2, 6 );
	str_print( d, TRUE );

	int g;
	str_copy( a, "d:\\lcc\\project\\bignum\\bigint.c" );
	// using two STRINGs
	g = str_extractpath( b, a );
	if (g==0)
		str_printf( NULL, "No path found!\n" );
	else
		str_print( b, TRUE );

	// using one STRING & an array of chars
	g = str_extractpath( b, "d:\\lcc\\project\\bignum\\bigint.c" );
	if (g==0)
		str_printf( NULL, "No path found!\n" );
	else
		str_print( b, TRUE );

	g = str_extractfile( b, "d:\\lcc\\project\\bignum\\bigint.c" );
	if (g==0)
		str_printf( NULL, "No file name found!\n" );
	else
		str_print( b, TRUE );

	str_resize( a, 2600 );

	str_insert( c, "The following example includes the important", 2, 30 );
	str_print( c, TRUE );

	int ret = str_resize(c, 0xFFFFFFFF);
	switch (ret){
	case STRERR_NONE:
		str_printc( "STRERR_NONE\n" );
		break;

	case STRERR_MEMORY:
		str_printc( "STRERR_MEMORY\n" );
		break;

	case STRERR_SIZE:
		str_printc( "STRERR_SIZE\n" );
		break;

	case STRERR_NOTFOUND:
		str_printc( "STRERR_NOTFOUND\n" );
		break;

	case STRERR_MAXALLOCAT:
		str_printc( "STRERR_MAXALLOCAT\n" );
		break;

	case STRERR_WRONGARGS:
		str_printc( "STRERR_WRONGARGS\n" );
		break;

	}

	str_format( a, "%s %lld", "The sections that follow demonstrate how the ", (long long) 10 );
	str_print( a, TRUE );

	str_printf( NULL, "%d\n", str_locatei( a, "that" ) );

	str_copy( b , "123456789");
	str_extract( b , a, 4, 8 );
	str_print( b, TRUE );

	str_copy(a , "d:\\lcc\\project\\bignum\\bigint.c" );
	str_extractpath( b, a );
	str_print( b, TRUE );
	str_extractfile( b, a );
	str_print( b, TRUE );

	str_printf( b, " %d\n", str_rinstr(b, 'n', str_len( b ) ) );

	str_printf( NULL, "str_lasterror %d\n", str_lasterror() );

	str_print( c, TRUE );

	str_format( a, "fmt %s%d", "The sections that follow demonstrate how the ", 78 );

	str_print( a, TRUE );

	str_insert( a, d, 2, str_len(d));

	str_print( a, TRUE );

	str_insert( a, "QW", 10, 2 );

	str_print( a, TRUE );

	int j;

	str_format( a, "fmt %s %d", "In the end he just carried on singing", 78 );

	str_print( a, TRUE );

	str_printf( NULL, "%d\n", str_locate( a, "end") );

	str_printf( NULL, "%d\n", str_locatei( a, "End") );

	// demo of str_xlate. first make a control array
	char x[256];
	int i;
	for ( i = 0; i<256; i++ )
  		x[i]= i;

	for ( i = 65; i<91; i++ )
  		x[i]= (i + 32);

	for ( i = 97; i<123; i++ )
  		x[i]= (i - 32);

	char test[] = "A Quick Brown Fox Jumps Over The Lazy Dog";
	str_copy( a, test );

	str_xlate( a, x );
	// all uppercase to lowercase & all lowercase to upercase
	str_print( a, TRUE );

	str_xlate( test, x );
	str_printc( test );

	int id;
	str_printf( NULL, "%d ", str_getid( a ) );
	str_printf( NULL, "%d ", str_getid( b ) );
	str_printf( NULL, "%d ", str_getid( c ) );
	str_printf( NULL, "%d\n", str_getid( d ) );
	str_delete( b );
	str_printf( NULL, "%d ", str_getid( a ) );
	str_printf( NULL, "%d ", str_getid( c ) );
	str_printf( NULL, "%d\n", str_getid( d ) );

	id = str_getid( c );
	str_print( c, TRUE );
	id = str_count();
	STRING hcpy = str_idreap( id );
	str_print( hcpy, TRUE );
	id = str_count();
	hcpy = str_idreap( id );
	str_print( hcpy, TRUE );

	str_printf( NULL, "%d\n", str_getid( c ));

	// convert tabs to spaces
	STRING q = str_new( "A\tquick\tbrown\tfox" );
	str_printf( q, " [num tabs %d]\n", str_tab_to_space( q, 4 ));

	// choose a file to open
//	FILE *f = fopen( "c:\\test.txt", "r" );
//	if (f){
//		int ret = str_fgets( f, q, 70 );
//		str_printf( q, "%d", ret );
//		fclose( f );
//	}
	str_printf( q, " str_hash  %u\n", str_hash( q ) );
	str_printf( q, " str_hashget  %u\n", str_hashget( q ) );

	char tst[] = "A    quick    brown    fox";
	str_printf(NULL, "Hash for array of chars %u\n", str_hash( tst ) );

	str_delete_all();

	// demo of qsort
	STRING s[4];
	s[0] = str_new( "tiger" );
	s[1] = str_new( "Hippopotamus" );
	s[2] = str_new( "Elephant" );
	s[3] = str_new( "aardvark" );
	str_printf( NULL, "\n");
	str_printf( s[0], " %d\n", str_getid( s[0] ) );
	str_printf( s[1], " %d\n", str_getid( s[1] ) );
	str_printf( s[2], " %d\n", str_getid( s[2] ) );
	str_printf( s[3], " %d\n", str_getid( s[3] ) );
	str_printf( NULL, "\n" );
	str_qsorti( &s[0], 4 );
	str_printf( s[0], " %d\n", str_getid( s[0] ) );
	str_printf( s[1], " %d\n", str_getid( s[1] ) );
	str_printf( s[2], " %d\n", str_getid( s[2] ) );
	str_printf( s[3], " %d\n", str_getid( s[3] ) );

	// demo of str_printf
	a = str_new( "The cat's age was " );
	str_printf( a, "%d\n", 54 );

	int pos = 3, index = 2;
	if ( str_search( "popo", &index, &pos ) ){
		str_printf( str_idreap(index), " Index - %d Position - %d\n", index, pos);
	}else{
		str_printc( "Not found!" );
	}

	pos = 3, index = 2;
	if ( str_searchi( "POPO", &index, &pos ) ){
		str_printf( str_idreap(index), " Index - %d Position - %d\n", index, pos);
	}else{
		str_printc( "Not found!" );
	}

	// get the last string object ID
	id = str_count();
	hcpy = str_idreap( id );
	str_print ( hcpy );

	// choose a file to append
//	FILE *f = fopen( "c:\\test.txt", "a" );
//	str_fputs( f, a );
//	fclose(f);
	str_hash( a );
	STRING jj = str_dup( a );
	str_print ( jj );

	str_delete_all();
	str_printf(NULL, "%d\n", str_count());
//	str_libvers();

	char string[12];
	printf("\nPress <Enter>\n");
	fgets(string, 8, stdin);


	return 0;

}




